from rest_framework.routers import DefaultRouter


class SuffixlessRouter(DefaultRouter):
    """
    So far we have never used this feature and it just makes our life harder, so we disable it
    """

    include_format_suffixes = False



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

from .settings import *  # NOQA: configure


class TgReactConfig(AppConfig):
    name = "tg_react"
    verbose_name = _("Tg react")



from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from django.utils.module_loading import import_string


# fields unconditionally excluded from user signup data
USER_SIGNUP_SKIPPED_FIELDS = {
    "id",
    "is_staff",
    "is_superuser",
    "is_active",
    "date_joined",
    "last_login",
}


def get_user_signup_fields():
    return getattr(
        settings,
        "TGR_USER_SIGNUP_FIELDS",
        [
            "name",
        ],
    )


def get_signup_skipped_fields():
    fields = getattr(settings, "TGR_USER_SIGNUP_SKIPPED_FIELDS", [])
    return USER_SIGNUP_SKIPPED_FIELDS.union(fields)


def get_email_case_sensitive():
    return getattr(
        settings, "TGR_EMAIL_CASE_SENSITIVE", True
    )  # True by default to maintain compatibility


def exclude_fields_from_user_details():
    return getattr(settings, "TGR_EXCLUDED_USER_FIELDS", [])


def get_user_extra_fields(validate=False):
    fields = getattr(settings, "TGR_USER_EXTRA_FIELDS", {})

    if validate:
        if not isinstance(fields, dict):
            raise ImproperlyConfigured("settings.TGR_USER_EXTRA_FIELDS must be a dict")

    res = {}
    for name, conf in fields.items():
        if validate and not isinstance(conf, (str, list)):
            raise ImproperlyConfigured(
                "settings.TGR_USER_EXTRA_FIELDS value must be a module path or list[path, kwargs]"
            )

        path = conf if not isinstance(conf, list) else conf[0]
        kwargs = {} if not isinstance(conf, list) else conf[1] or {}

        res[name] = [import_string(path), kwargs]

    return res


def get_password_recovery_url():
    return getattr(settings, "TGR_PASSWORD_RECOVERY_URL", "/reset_password/%s")


def get_post_login_handler():
    return getattr(settings, "TGR_POST_LOGIN_HANDLER", None)


def get_post_logout_handler():
    return getattr(settings, "TGR_POST_LOGOUT_HANDLER", None)


def configure():
    if not isinstance(exclude_fields_from_user_details(), (list, tuple)):
        raise ImproperlyConfigured(
            "settings.TGR_EXCLUDED_USER_FIELDS must be list|tuple"
        )

    if not isinstance(get_user_signup_fields(), (list, tuple)):
        raise ImproperlyConfigured("settings.TGR_USER_SIGNUP_FIELDS must be list|tuple")

    handler = get_post_login_handler()
    if handler is not None and not isinstance(handler, str):
        raise ImproperlyConfigured(
            "settings.TGR_POST_LOGIN_HANDLER must be module path"
        )

    handler = get_post_logout_handler()
    if handler is not None and not isinstance(handler, str):
        raise ImproperlyConfigured(
            "settings.TGR_POST_LOGOUT_HANDLER must be module path"
        )

    recovery_url = get_password_recovery_url()
    if not isinstance(recovery_url, str):
        raise ImproperlyConfigured("settings.TGR_PASSWORD_RECOVERY_URL must be str")

    try:
        recovery_url % "TEST"
    except TypeError:
        raise ImproperlyConfigured(
            "settings.TGR_PASSWORD_RECOVERY_URL must contain a string "
            "formatting token for base64 encoded data"
        )

    get_user_extra_fields(validate=True)


configure()



from django.conf import settings
from django.utils.decorators import method_decorator
from django.utils.module_loading import import_string
from django.views.decorators.csrf import ensure_csrf_cookie
from rest_framework import generics, status
from rest_framework.authentication import SessionAuthentication
from rest_framework.permissions import (
    AllowAny,
    IsAuthenticated,
    IsAuthenticatedOrReadOnly,
)
from rest_framework.response import Response
from rest_framework.views import APIView

from django.contrib.auth import get_user_model
from django.utils.translation import gettext as _, get_language_from_request

from .serializers import (
    AuthenticationSerializer,
    UserDetailsSerializer,
    SignupSerializer,
    ForgotPasswordSerializer,
    RecoveryPasswordSerializer,
    LanguageCodeSerializer,
)

# for email notifications
from django.template.loader import get_template

from tg_react.settings import (
    get_password_recovery_url,
    get_post_login_handler,
    get_post_logout_handler,
    get_signup_skipped_fields,
)


def do_login(request, user):
    if hasattr(request, "session"):
        old_session = request.session.session_key

    else:
        old_session = None

    from django.contrib.auth import login  # NOQA

    login(request, user)

    post_login = get_post_login_handler()
    if post_login:
        post_login = import_string(post_login)

        post_login(user=user, request=request, old_session=old_session)


def do_logout(request):
    if hasattr(request, "session"):
        old_session = request.session.session_key

    else:
        old_session = None

    old_user = request.user

    from django.contrib.auth import logout  # NOQA

    logout(request)

    post_logout = get_post_logout_handler()
    if post_logout:
        post_logout = import_string(post_logout)

        post_logout(user=old_user, request=request, old_session=old_session)


class UnsafeSessionAuthentication(SessionAuthentication):
    def enforce_csrf(self, request):
        pass


class UserDetails(generics.RetrieveUpdateAPIView):
    serializer_class = UserDetailsSerializer
    authentication_classes = (SessionAuthentication,)
    permission_classes = (IsAuthenticatedOrReadOnly,)

    def get_object(self):
        return self.request.user

    @method_decorator(ensure_csrf_cookie)
    def get(self, request, *args, **kwargs):
        user = self.get_object()
        if user.is_authenticated:
            return super().get(request, *args, **kwargs)

        return Response({"authenticated": False}, status=status.HTTP_401_UNAUTHORIZED)

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop("partial", False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        if serializer.is_valid():
            self.perform_update(serializer)
            return Response(serializer.data)

        return Response(
            {"errors": serializer.errors}, status=status.HTTP_400_BAD_REQUEST
        )


class AuthenticationView(APIView):
    class UnsafeSessionAuthentication(SessionAuthentication):
        def enforce_csrf(self, request):
            pass

    throttle_classes = ()
    permission_classes = (AllowAny,)
    authentication_classes = (UnsafeSessionAuthentication,)
    serializer_class = AuthenticationSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            do_login(request, serializer.user)

            return Response({"success": True})

        return Response(
            {"errors": serializer.errors}, status=status.HTTP_400_BAD_REQUEST
        )


class SetLanguageView(generics.RetrieveUpdateAPIView):

    throttle_classes = ()
    permission_classes = (AllowAny,)
    authentication_classes = (UnsafeSessionAuthentication,)
    serializer_class = LanguageCodeSerializer

    @property
    def raw_request(self):
        return self.request._request  # pylint: disable=protected-access

    def get_object(self):
        return {
            "language_code": getattr(
                self.raw_request, "_lang", get_language_from_request(self.raw_request)
            )
        }

    def update(self, request, *args, **kwargs):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            setattr(
                self.raw_request,
                "update_language_cookie",
                serializer.validated_data["language_code"],
            )
            setattr(
                self.raw_request, "_lang", serializer.validated_data["language_code"]
            )

            return self.get(request, *args, **kwargs)

        return Response(
            {"errors": serializer.errors}, status=status.HTTP_400_BAD_REQUEST
        )


class LogoutView(APIView):
    throttle_classes = ()
    permission_classes = (IsAuthenticated,)
    authentication_classes = (SessionAuthentication,)

    def post(self, request):
        do_logout(request)

        return Response({"success": True})


class SignUpView(APIView):
    serializer_class = SignupSerializer
    permission_classes = (AllowAny,)
    authentication_classes = (UnsafeSessionAuthentication,)

    def post(self, request):
        serializer = self.serializer_class(
            data=request.data, context={"request": request}
        )
        if serializer.is_valid():
            data = serializer.validated_data.copy()
            password = data.pop("password", None)

            for skipped_field in get_signup_skipped_fields():
                data.pop(skipped_field, None)

            user = get_user_model()(**data)
            user.set_password(password)
            user.save()

            from django.contrib.auth import authenticate  # NOQA

            user = authenticate(email=data["email"], password=password)

            do_login(request, user)

            return Response({"success": True})

        return Response(
            {"errors": serializer.errors}, status=status.HTTP_400_BAD_REQUEST
        )


class ForgotPassword(APIView):
    """
    Initiate a password restore procedure.
    """

    # Example request data
    # {
    #   "email" : "somename@somedomain.com"
    # }

    serializer_class = ForgotPasswordSerializer
    authentication_classes = (UnsafeSessionAuthentication,)
    permission_classes = (AllowAny,)

    def send_email_notification(self, user, uid_and_token_b64):
        # define email notification logic
        from django.core.mail import EmailMultiAlternatives  # NOQA

        # make confirm reset url
        path = get_password_recovery_url() % uid_and_token_b64
        confirm_reset_url = settings.SITE_URL + path

        subject = _("Password restore")
        context = {"user": user, "confirm_reset_url": confirm_reset_url}
        text_content = get_template("emails/password_reset.txt").render(context)
        html_content = get_template("emails/password_reset.html").render(context)

        msg = EmailMultiAlternatives(subject, text_content, to=[user.email])
        msg.attach_alternative(html_content, "text/html")
        msg.send()

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            self.send_email_notification(
                serializer.user, serializer.validated_data["uid_and_token_b64"]
            )
            return Response({"success": True})

        return Response(
            {"errors": serializer.errors}, status=status.HTTP_400_BAD_REQUEST
        )


class RestorePassword(APIView):
    """
    Validate token and change a user password.
    """

    # Example request data
    # {
    #   "password" : "asdasd",
    #   "password_confirm" : "asdasd",
    #   "uid_and_token_b64" : "eyJ1aWQiOiAxLCAidG9rZW4iOiAiNDN1LWY5ZDIyMmM4YTNkZDhmYjI2OThlIn0="
    # }

    serializer_class = RecoveryPasswordSerializer
    authentication_classes = (UnsafeSessionAuthentication,)
    permission_classes = (AllowAny,)

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            user = serializer.user
            user.set_password(serializer.validated_data["password"])
            user.save()
            return Response({"success": True})

        return Response(
            {"errors": serializer.errors}, status=status.HTTP_400_BAD_REQUEST
        )



import json
import base64

from rest_framework import serializers
from rest_framework.utils.field_mapping import ClassLookupDict

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.tokens import default_token_generator
from django.utils.translation import gettext as _

from tg_react.settings import (
    exclude_fields_from_user_details,
    get_user_signup_fields,
    get_email_case_sensitive,
    get_user_extra_fields,
)


class UserDetailsSerializer(serializers.ModelSerializer):
    # Overriding default field to get rid of existing uniquevalidator
    # because i want to show better validation message (see validate_email below)
    email = serializers.EmailField(validators=[])

    class Meta:
        model = get_user_model()
        fields = [
            f.name
            for f in get_user_model()._meta.fields
            if f.name not in exclude_fields_from_user_details()
        ]

        extra_kwargs = {
            "password": {"write_only": True},
            "id": {"read_only": True},
            "is_active": {"read_only": True},
            "is_staff": {"read_only": True},
            "is_superuser": {"read_only": True},
            "date_joined": {"read_only": True},
            "last_login": {"read_only": True},
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        for field in exclude_fields_from_user_details():
            if field in self.fields:
                del self.fields[field]

    def to_representation(self, instance):
        data = super().to_representation(instance)

        if not get_email_case_sensitive():
            email = data and data.get("email")
            if email:
                data["email"] = email.lower()

        return data

    def validate_email(self, data):
        current_email = self.instance.email

        if not get_email_case_sensitive():
            data = data.lower()
            current_email = current_email.lower()

        if (
            get_user_model().objects.filter(email=data).exists()
            and current_email != data
        ):
            raise serializers.ValidationError(
                _("User with this e-mail address already exists.")
            )

        return data

    def get_fields(self):
        static_fields = super().get_fields()

        for name, cfg in get_user_extra_fields().items():
            static_fields[name] = cfg[0](**cfg[1])

        return static_fields


class AuthenticationSerializer(serializers.Serializer):
    password = serializers.CharField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields[get_user_model().USERNAME_FIELD] = serializers.CharField()

        self.user = None

    def validate(self, attrs):
        credentials = {"password": attrs.get("password", None)}

        username_field = get_user_model().USERNAME_FIELD
        credentials[username_field] = attrs.get(username_field, None)

        if all(credentials.values()):
            from django.contrib.auth import authenticate  # NOQA

            if username_field == "email":
                if not get_email_case_sensitive():
                    credentials[username_field] = credentials[username_field].lower()

            user = authenticate(**credentials)

            if user:
                if not user.is_active:
                    raise serializers.ValidationError(
                        _("Your account has been disabled.")
                    )

                self.user = user
                # hide the password so it wont leak
                credentials["password"] = "-rr-"

                return credentials

            raise serializers.ValidationError(
                _("Unable to login with provided credentials.")
            )

        raise serializers.ValidationError(_("Please enter both email and password."))

    def create(self, validated_data):
        return validated_data


def phonenumber_validation(data):
    """Validates phonenumber

    Similar to phonenumber_field.validators.validate_international_phonenumber() but uses a different message if the
    country prefix is absent.
    """
    from phonenumber_field.phonenumber import to_python  # NOQA

    phone_number = to_python(data)
    if not phone_number:
        return data

    if not phone_number.country_code:
        raise serializers.ValidationError(
            _("Phone number needs to include valid country code (E.g +37255555555).")
        )

    if not phone_number.is_valid():
        raise serializers.ValidationError(_("The phone number entered is not valid."))

    return data


class SignupSerializer(serializers.Serializer):
    email = serializers.EmailField()
    password = serializers.CharField()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        fields_we_do_not_want = [
            "id",
            "email",
            "password",
            "is_staff",
            "is_superuser",
            "is_active",
            "date_joined",
            "last_login",
        ]
        important_signup_fields = get_user_signup_fields()
        model = get_user_model()
        # Using ModelSerializers model field to serializer field mapper
        # to build missing fields for signup serializer
        original_mapping = serializers.ModelSerializer.serializer_field_mapping

        try:
            from phonenumber_field.modelfields import PhoneNumberField  # NOQA

            original_mapping.update({PhoneNumberField: serializers.CharField})
        except ImportError:
            pass

        mapping = ClassLookupDict(original_mapping)
        for model_field in model._meta.fields:
            if (
                model_field.name in important_signup_fields
                and model_field.name not in fields_we_do_not_want
            ):
                field_kwargs = {"required": True}
                if "phone" in model_field.name:
                    field_kwargs["validators"] = [phonenumber_validation]
                self._declared_fields[model_field.name] = mapping[model_field](
                    **field_kwargs
                )

    def validate_email(self, data):
        if not get_email_case_sensitive():
            data = data.lower()

        if get_user_model().objects.filter(email=data).exists():
            raise serializers.ValidationError(
                _("User with this e-mail address already exists.")
            )

        return data


class ForgotPasswordSerializer(serializers.Serializer):
    email = serializers.EmailField(required=True)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.user = None

    def validate_email(self, email):
        user_model = get_user_model()

        if not get_email_case_sensitive():
            email = email.lower()

        try:
            self.user = user_model.objects.get(email=email)
        except user_model.DoesNotExist:
            raise serializers.ValidationError(
                _("We do not have user with given e-mail address in our system.")
            )

        return email

    def validate(self, attrs):
        # Serialize uid and token to json then encode to base64
        uid_and_token = json.dumps(
            {
                "uid": self.user.pk,
                "token": default_token_generator.make_token(self.user),
            }
        ).encode("utf-8")
        return {
            "uid_and_token_b64": base64.urlsafe_b64encode(uid_and_token).decode("ascii")
        }


class RecoveryPasswordSerializer(serializers.Serializer):
    password = serializers.CharField(required=True)
    password_confirm = serializers.CharField(required=True)

    # uid_and_token receive json dict encoded with base64
    uid_and_token_b64 = serializers.CharField(required=True)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.user = None

    def validate(self, attrs):
        if attrs["password"] != attrs["password_confirm"]:
            raise serializers.ValidationError(_("Password mismatch."))

        return {"password": attrs["password"]}

    def validate_uid_and_token_b64(self, uid_and_token_b64):

        try:
            # Deserialize data from json
            json_data = base64.urlsafe_b64decode(uid_and_token_b64).decode("utf-8")
            data = json.loads(json_data)
        except Exception:
            raise serializers.ValidationError(_("Broken data."))

        uid = data.get("uid", None)
        token = data.get("token", None)

        try:
            assert uid and token and isinstance(uid, int)
        except AssertionError:
            raise serializers.ValidationError(_("Broken data."))

        user_model = get_user_model()

        try:
            self.user = user_model.objects.get(pk=uid)
        except user_model.DoesNotExist:
            raise serializers.ValidationError(_("User not found."))

        # validate token
        if not default_token_generator.check_token(self.user, token):
            msg = "%s %s" % (
                _(
                    "This password recovery link has expired or associated user does not exist."
                ),
                _("Use password recovery form to get new e-mail with new link."),
            )
            raise serializers.ValidationError(msg)


class LanguageCodeSerializer(serializers.Serializer):
    language_code = serializers.ChoiceField(choices=settings.LANGUAGES, required=True)



from django.conf.urls import re_path

from .views import (
    UserDetails,
    AuthenticationView,
    LogoutView,
    SignUpView,
    SetLanguageView,
)
from .views import ForgotPassword
from .views import RestorePassword


urlpatterns = [
    re_path(r"^me$", UserDetails.as_view(), name="api-user-details"),
    re_path(r"^login$", AuthenticationView.as_view(), name="api-user-login"),
    re_path(r"^lang$", SetLanguageView.as_view(), name="api-user-language"),
    re_path(r"^logout$", LogoutView.as_view(), name="api-user-logout"),
    # signup
    re_path(r"^signup$", SignUpView.as_view(), name="api-signup"),
    # password recovery
    re_path(r"^forgot_password$", ForgotPassword.as_view(), name="api-forgot-password"),
    re_path(
        r"^forgot_password/token$",
        RestorePassword.as_view(),
        name="api-forgot-password-token",
    ),
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tg_react', 'tg_react.api', 'tg_react.api.accounts']

package_data = \
{'': ['*'],
 'tg_react': ['locale/en/LC_MESSAGES/*',
              'locale/en_US/LC_MESSAGES/*',
              'locale/et/LC_MESSAGES/*',
              'locale/ru/LC_MESSAGES/*',
              'templates/tg_react_emails/*']}

install_requires = \
['django', 'djangorestframework>=3.6.4']

setup_kwargs = {
    'name': 'tg-react',
    'version': '3.0.0b0',
    'description': 'Helpers for react based applications running on django.',
    'long_description': '# tg-react\n\n![https://badge.fury.io/py/tg-react](https://badge.fury.io/py/tg-react.png)\n![Workflow status](https://github.com/thorgate/tg-react/actions/workflows/python-package.yml/badge.svg?branch=master)\n\nHelpers for react based applications running on django.\n\n## Documentation\n\nThe full documentation is at https://tg-react.readthedocs.org.\n\n## Quickstart\n\nInstall tg-react:\n\n```\npip install tg-react\n```\n\nThen use it in a project:\n\n```\nimport tg_react\n```\n\n## Features\n\n* TODO\n\n.. TODO: List features and link to documentation for more information\n\n## Changelog\n\nChanges are documented under `Github Releases <https://github.com/thorgate/tg-react/releases>`_.\n',
    'author': 'Thorgate',
    'author_email': 'jyrno@thorgate.eu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/thorgate/tg-react',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4',
}


setup(**setup_kwargs)



from django.conf import settings
from django.utils import translation
from django.utils.cache import patch_vary_headers


class LocaleMiddleware:
    """
    This is a very simple middleware that parses a request
    and decides what translation object to install in the current
    thread context depending on the selected language.

    This also allows us to update the language cookie whenever our api endpoint is used.
    """

    def get_language_for_user(self, request):
        if request.user.is_authenticated() and hasattr(request.user, "language"):
            return getattr(request.user, "language")

        return translation.get_language_from_request(request)

    def process_request(self, request):
        if request.META.get("HTTP_ACCEPT_LANGUAGE", None) is not None:
            del request.META["HTTP_ACCEPT_LANGUAGE"]

        translation.activate(self.get_language_for_user(request))
        request.LANGUAGE_CODE = translation.get_language()

    def process_response(self, request, response):
        language = getattr(request, "update_language_cookie", False)

        if language:
            if request.user.is_authenticated() and hasattr(request.user, "language"):
                language = getattr(request.user, "language")

            else:
                language = language or request.LANGUAGE_CODE

            response.set_cookie(
                settings.LANGUAGE_COOKIE_NAME,
                language,
                max_age=settings.LANGUAGE_COOKIE_AGE,
                path=settings.LANGUAGE_COOKIE_PATH,
                domain=settings.LANGUAGE_COOKIE_DOMAIN,
            )

        patch_vary_headers(response, ("Accept-Language",))
        response["Content-Language"] = translation.get_language()
        translation.deactivate()
        return response
